<?php
namespace OCA\Dematpayslip\Db;

use OCP\IDbConnection;
use OCP\AppFramework\Db\QBMapper;

use OCP\DB\QueryBuilder\IQueryBuilder;

class DematpayslipArchiveMapper extends QBMapper {

    public function __construct(IDbConnection $db) {
        parent::__construct($db, 'dematpayslip_archive', DematpayslipArchive::class);
    }

    public function find(int $id) {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName())
            ->where(
                $qb->expr()->eq('id', $qb->createNamedParameter($id))
            );

        return $this->findEntity($qb);
    }

    public function findAllByIdFile(int $idFile) {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName())
            ->where(
                $qb->expr()->eq('id_file', $qb->createNamedParameter($idFile))
            );

        return $this->findEntities($qb);
    }

    public function findAllByFilePath(string $filePath) {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName())
            ->where(
                $qb->expr()->eq('file_path', $qb->createNamedParameter($filePath))
            );

        return $this->findEntities($qb);
    }

    public function findAllByAgentAndPeriod(string $siret, string $matricule, string $lastname, string $firstname, string $start_date, string $end_date) {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName())
            ->where(
                $qb->expr()->eq('siret', $qb->createNamedParameter($siret))
            )
            ->andWhere(
                $qb->expr()->eq('matricule', $qb->createNamedParameter($matricule))
            )
            ->andWhere(
                $qb->expr()->eq('lastname', $qb->createNamedParameter($lastname))
            )
            ->andWhere(
                $qb->expr()->eq('firstname', $qb->createNamedParameter($firstname))
            )
            ->andWhere(
                $qb->expr()->eq('start_date', $qb->createNamedParameter($start_date))
            )
            ->andWhere(
                $qb->expr()->eq('end_date', $qb->createNamedParameter($end_date))
            )
            ->andWhere(
                $qb->expr()->gte('status', $qb->createNamedParameter(DematpayslipArchive::STATUS_INITIALIZED))
            );;

        return $this->findEntities($qb);
    }

    public function findAllNotDeleted() {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*');
        $qb->from($this->getTableName());

        if (!empty($statusList)) {
            $qb->where(
                $qb->expr()->notIn('status', $qb->createNamedParameter([DematpayslipArchive::STATUS_DELETED], IQueryBuilder::PARAM_STR_ARRAY))
            );
        }

        return $this->findEntities($qb);
    }

    public function findAllByStepStatus(int $stepStatus) {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*');
        $qb->from($this->getTableName());

        $statusList = DematpayslipArchive::getStepStatusListFrom($stepStatus);
        if (!empty($statusList)) {
            $qb->where(
                $qb->expr()->in('status', $qb->createNamedParameter($statusList, IQueryBuilder::PARAM_STR_ARRAY))
            );
        }

        return $this->findEntities($qb);
    }

    public function findAll($deleted = 0) {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName());

        if ($deleted == 0) {
            $qb->where(
                $qb->expr()->notIn('status', $qb->createNamedParameter([DematpayslipArchive::STATUS_DELETED], IQueryBuilder::PARAM_STR_ARRAY))
            );
        } else if ($deleted > 0) {
            $qb->where(
                $qb->expr()->in('status', $qb->createNamedParameter([DematpayslipArchive::STATUS_DELETED], IQueryBuilder::PARAM_STR_ARRAY))
            );
        }

        return $this->findEntities($qb);
    }
}
